from concurrent.futures import ThreadPoolExecutor, as_completed
from ipaddress import ip_address, IPv4Address
from random import shuffle
from urllib.request import urlopen

from .commandparams import Param, ValidationError
from . import debug, error


ip_adr_getters = [
    'api.ipify.org',
    'api.my-ip.io/ip',
    'ident.me',
]


def get_text(url, timeout=1):
    try:
        with urlopen(f'https://{url}', timeout=timeout) as resp:
            if resp.status == 200:
                txt = resp.read().decode().strip()
                return True, txt
            else:
                reason = f'{url} -> {resp.status} {resp.reason}'
    except Exception as x:
        reason = f'{url} ! {type(x)} {x}'
    return False, reason


def get_ip_adr_sync():
    shuffle(ip_adr_getters)
    for url in ip_adr_getters:
        ok, result = get_text(url)
        if ok:
            debug(f'ip adr : {result}')
            return True, result
        else:
            debug(f'{url} ! {result}')
    error('unabled to get ip adr')
    return False, 'ERROR'


def get_ip_adr_sub():
    n = len(ip_adr_getters)
    with ThreadPoolExecutor(max_workers=n) as ex:
        futures = [ex.submit(get_text, url) for url in ip_adr_getters]
        for future in as_completed(futures):
            ok, result = future.result()
            if ok:
                return True, result
    error('unabled to get ip adr')
    return False, 'ERROR'


def get_ip_adr_map():
    with ThreadPoolExecutor(
        max_workers = len(ip_adr_getters)
    ) as ex:
        for ok, result in ex.map(
            get_text,
            ip_adr_getters,
        ):
            if ok:
                return True, result
    error('unabled to get ip adr')
    return False, 'ERROR'


class ParamIPv4(Param):
    name = 'IPv4 adr'

    def verify(self, value):
        try:
            adr = ip_address(value)
            if not isinstance(adr, IPv4Address):
                raise ValueError
            return adr
        except ValueError:
            raise ValidationError(f'{value} is not IPv4')

